#!/bin/sh
#
# This is a crude script to watch the calls table and lookup the number
# in the cid table. If the number is found fetch the information from
# the people table. Finally print what information was found.
#
# By Lorance Stinson LoranceStinson AT gmail DOT com

# The path to the shql database.
dbdir="$HOME/shql/addr"

#**************************************************************************
# Defaults.
#**************************************************************************
ringbell=""     # Ring the terminal bell before each call is printed.
clearscreen=""  # Clear the screen before each call is printed.
emailto=""      # Email each call to this address.
printcalls="Y"  # Print the calls to the terminal.
runoncall=""    # A command to run when a call arrives.
CALL=""         # The call data is stored here.
export CALL

#**************************************************************************
# usage() - Display the command line help text.
#**************************************************************************
usage(){
    if [ "$1" ] ; then
        echo "$1"
        echo ""
    fi
    cat<<EOHELP
Usage:  `basename $0` [options]
Watched the table calls in an shql database waiting for new calls.
When a new call appears in the table it's information is displayed.
Options:
    -b          Ring the terminal bell before each call is printed.
    -c          Clear the screen before each call is printed.
    -d <dir>    Use dir for the database directory.
                Defaults to: $dbdir
    -e <addr>   Email the call to the email address.
    -q          Do not print the calls to the terminal.
    -r command  Run command when a call arrives.
                The commmand will be run with the sh shell.
                The call is stored in the environment variable \$CALL
EOHELP
    exit 1
}

#**************************************************************************
# Check and handle commandline options.
#**************************************************************************
while [ "$#" -gt 0 ] ; do
    case "$1" in
        # Ring the terminal bell.
        -b) ringbell="Y"; shift;;
        # Clear the screen.
        -c) clearscreen="Y"; shift;;
        # Alternate DB dir.
        -d) [ "$2" ] && dbdir="$2"      || usage "$1 requires an argument."
            shift 2;;
        # Email the call.
        -e) [ "$2" ] && emailto="$2"    || usage "$1 requires an argument."
            shift 2;;
        # Print the help text.
        -h) usage;;
        # Don't print the call.
        -q) printcalls=""; shift;;
        # Run a command whena call comes in.
        -r) [ "$2" ] && runoncall="$2"  || usage "$1 requires an argument."
            shift 2;;
        -*) usage "Invalid option $1";;
    esac
done

# Make sure the calls table exists.
[ ! -r "$dbdir/calls~" ] &&
    echo "Could not find the calls table in the database." &&
    echo "Please make sure $dbdir is actually an shql database." &&
    exit 1

tail -f -n 0 $dbdir/calls~ |
while read line ; do
    [ "$clearscreen" ] && clear
    [ "$ringbell" ] && printf "\a"
    # Attempt to find a match in the cid and people tables.
    # Print any CID and/or personal data found.
    CALL=`echo "$line" | awk -F"	" -v dbdir="$dbdir" \
        'NR == 1 {
            # Save the CID information.
            number  = $1
            name    = $2
            date = substr($3,1,2) "-" substr($3,3)
            time = substr($4,1,2) ":" substr($4,3)
            systime = $5
            # Search for an entry in the cid table.
            search = "grep \"	" number "	\" " dbdir "/cid~" 
            if (search | getline) {
                person = $1
                note = $3
            }
            printf "System Time: %s\n", systime
            printf "CID Name:    %s\n", name
            printf "CID Number:  %s\n", number
            printf "CID Date:    %s\n", date
            printf "CID Time:    %s\n", time
            if (person) {
                # Get the persons information.
                search = "grep \"^" person "\" " dbdir "/people~" 
                if (search | getline) {
                    if ($1)   printf "DB Name:     %s\n", $1
                    if ($2)   printf "DB Email:    %s\n", $2
                    if ($3)   printf "DB AltEmail: %s\n", $3
                    if ($4)   printf "DB Home:     %s\n", $4
                    if ($5)   printf "DB Cell:     %s\n", $5
                    if ($6)   printf "DB Work:     %s\n", $6
                    if (note) printf "DB Note:     %s\n", note
                } else {
                    print "Error: Unable to locate entry for: " person
                }
            } else {
                print "No database entry found."
            }
        }' -`
        [ "$printcalls" ] && echo "$CALL" && echo ""
        [ "$emailto" ] && echo "$CALL" | mail -s "Phone Call `date`" $emailto
        [ "$runoncall" ] && eval $runoncall
done
